
use std::io;
use std::fs::read_dir;

pub struct Walker {
	root: String
}

impl Walker {
	pub fn new(path: &str) -> Self {
		let root = path.to_owned();
		Self {root}
	}

	/// Ejemplo
	pub fn get_entries_path(&self) -> io::Result<impl Iterator<Item=String>> {
		let entries = read_dir(&self.root)?;
		let iter = entries
			.flatten() // Option<DirEntry> -> DirEntry
			.map(|entry| entry.path()) // entry -> PathBuf
			.map(|path| path.to_str().unwrap().to_owned() ); //PathBuf -> String
		Ok(iter)
	}

	// Utilizo filtro para quedarme con los directorios
	pub fn get_directories_path(&self) -> io::Result<impl Iterator<Item=String>> {
		let entries = read_dir(&self.root)?;
		let iter = entries
			.flatten() // Option<DirEntry> -> DirEntry
			.filter(entry_is_dir)
			.map(|entry| entry.path()) // entry -> PathBuf
			.map(|path| path.to_str().unwrap().to_owned() ); //PathBuf -> String
		Ok(iter)
	}
}

fn entry_is_dir(dir: &std::fs::DirEntry) -> bool {
	dir.file_type().unwrap().is_dir()
}

#[test]
fn iter_entries() {
	let walker = Walker::new("test-tree");
	let entries_iter = walker.get_entries_path().unwrap();
	let entries : Vec<String> = entries_iter.collect();
	assert_eq!(entries, ["test-tree/sub-tree", "test-tree/empty-file"]);
}


#[test]
fn iter_dirs() {
	let walker = Walker::new("test-tree");
	let entries_iter = walker.get_directories_path().unwrap();
	let entries : Vec<String> = entries_iter.collect();
	assert_eq!(entries, ["test-tree/empty-file"]);
}
